/*
 * Copyright (c) 1997  Motoyuki Kasahara
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef MAIN_H
#define MAIN_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <time.h>
#include <sys/types.h>

#include <eb/eb.h>
#include <eb/error.h>
#include <eb/text.h>

#include "readconf.h"
#include "permission.h"
#include "linebuf.h"
#include "defs.h"


/*
 * Book -- represents a book.
 */
typedef struct book {
    /*
     * Book code.
     */
    int code;

    /*
     * Name of the book.
     */
    char name[MAXLEN_BOOKNAME + 1];

    /*
     * Current subbook name (= directory name).
     */
    const char *subname;

    /*
     * Access permission to the current client.
     */
    int permflag;

    /*
     * Title of the book.
     */
    char title[MAXLEN_BOOKTITLE + 1];

    /*
     * The maximum number of connections to the book.
     */
    int max_clients;

    /*
     * Access permission list to the book.
     */
    Permission permissions;

    /*
     * Entity of the CD-ROM book.
     */
    EB_Book book;

    /*
     * Appendix entitiy of the CD-ROM book.
     */
    EB_Appendix appendix;

    /*
     * Refer to a next book.
     * Set to NULL when there is no more book.
     */
    struct book *next;
} Book;


/*
 * Function declarations.
 */
#ifdef __STDC__

/* book.c */
void initialize_book_registry(void);
void clear_book_registry(void);
void terminate_book_registry(void);
int add_book(void);
void delete_book(void);
int set_current_book(const char *);
void unset_current_book(void);
int set_current_subbook(const char *);
void unset_current_subbook(void);
Book *find_book(const char *);
void set_book_permissions(void);
/* ndtp.c */
int ndtp_main(void);
/* texthook.c */
void initialize_text_hookset(void);

#else /* not __STDC__ */

/* book.c */
void initialize_book_registry();
void clear_book_registry();
void terminate_book_registry();
int add_book();
void delete_book();
int set_current_book();
void unset_current_book();
int set_current_subbook();
void unset_current_subbook();
Book *find_book();
void set_book_permissions();
/* ndtp.c */
int ndtp_main();
/* texthook.c */
void initialize_text_hookset();

#endif /* not __STDC__ */


/*
 * Global variables.
 */
extern Configuration configuration_table[];
extern const char *invoked_name;
extern const char *program_name;
extern const char *program_version;
extern char work_path[];
extern char configuration_filename[];
extern char server_name[];
extern int listening_port;
extern int old_listening_port;
extern int listening_file;
extern int accepted_file;
extern uid_t user_id;
extern gid_t group_id;
extern char pid_filename[];
extern char old_pid_filename[];
extern int max_clients;
extern int connection_count;
extern int idle_timeout;
extern Permission identifications;
extern int identification_timeout;
extern Permission permissions;
extern Book *book_registry;
extern int book_count;
extern Book *current_book;
extern int max_hits;
extern EB_Hookset text_hookset;
extern char client_hostname[];
extern char client_address[];
extern char client_user[];
extern int server_mode;
extern int syslog_facility;
extern Line_Buffer line_buffer;
extern char alias_eiwa[];
extern char alias_waei[];
extern char alias_kojien[];

#endif /* not MAIN_H */
